@@ -2,142 +2,192 @@
 # See LICENSE for details.
 
 """
-Test cases for twisted.reflect module.
+Test cases for the L{twisted.python.reflect} module.
 """
 
-import weakref, os
-from ihooks import ModuleImporter
+from __future__ import division, absolute_import
 
-try:
-    from collections import deque
-except ImportError:
-    deque = None
+import os
+import weakref
+from collections import deque
 
+from twisted.python.compat import _PY3
 from twisted.trial import unittest
-from twisted.python import reflect, util
+from twisted.trial.unittest import SynchronousTestCase as TestCase
+from twisted.python import reflect
+from twisted.python.reflect import (
+    accumulateMethods, prefixedMethods, prefixedMethodNames,
+    addMethodNamesToDict, fullyQualifiedName)
 from twisted.python.versions import Version
 
 
-
-class SettableTest(unittest.TestCase):
-    def setUp(self):
-        self.setter = reflect.Settable()
-
-    def tearDown(self):
-        del self.setter
-
-    def testSet(self):
-        self.setter(a=1, b=2)
-        self.assertEqual(self.setter.a, 1)
-        self.assertEqual(self.setter.b, 2)
-
-
-
-class AccessorTester(reflect.Accessor):
-
-    def set_x(self, x):
-        self.y = x
-        self.reallySet('x', x)
-
-
-    def get_z(self):
-        self.q = 1
-        return 1
-
-
-    def del_z(self):
-        self.reallyDel("q")
-
-
-
-class PropertyAccessorTester(reflect.PropertyAccessor):
+class Base(object):
     """
-    Test class to check L{reflect.PropertyAccessor} functionalities.
-    """
-    r = 0
-
-    def set_r(self, r):
-        self.s = r
-
-
-    def set_x(self, x):
-        self.y = x
-        self.reallySet('x', x)
-
-
-    def get_z(self):
-        self.q = 1
-        return 1
-
-
-    def del_z(self):
-        self.reallyDel("q")
-
-
-
-class AccessorTest(unittest.TestCase):
-    def setUp(self):
-        self.tester = AccessorTester()
-
-    def testSet(self):
-        self.tester.x = 1
-        self.assertEqual(self.tester.x, 1)
-        self.assertEqual(self.tester.y, 1)
-
-    def testGet(self):
-        self.assertEqual(self.tester.z, 1)
-        self.assertEqual(self.tester.q, 1)
-
-    def testDel(self):
-        self.tester.z
-        self.assertEqual(self.tester.q, 1)
-        del self.tester.z
-        self.assertEqual(hasattr(self.tester, "q"), 0)
-        self.tester.x = 1
-        del self.tester.x
-        self.assertEqual(hasattr(self.tester, "x"), 0)
-
-
-
-class PropertyAccessorTest(AccessorTest):
-    """
-    Tests for L{reflect.PropertyAccessor}, using L{PropertyAccessorTester}.
+    A no-op class which can be used to verify the behavior of
+    method-discovering APIs.
     """
 
-    def setUp(self):
-        self.tester = PropertyAccessorTester()
-
-
-    def test_setWithDefaultValue(self):
+    def method(self):
         """
-        If an attribute is present in the class, it can be retrieved by
-        default.
+        A no-op method which can be discovered.
         """
-        self.assertEqual(self.tester.r, 0)
-        self.tester.r = 1
-        self.assertEqual(self.tester.r, 0)
-        self.assertEqual(self.tester.s, 1)
 
 
-    def test_getValueInDict(self):
+
+class Sub(Base):
+    """
+    A subclass of a class with a method which can be discovered.
+    """
+
+
+
+class Separate(object):
+    """
+    A no-op class with methods with differing prefixes.
+    """
+
+    def good_method(self):
         """
-        The attribute value can be overriden by directly modifying the value in
-        C{__dict__}.
+        A no-op method which a matching prefix to be discovered.
         """
-        self.tester.__dict__["r"] = 10
-        self.assertEqual(self.tester.r, 10)
 
 
-    def test_notYetInDict(self):
+    def bad_method(self):
         """
-        If a getter is defined on an attribute but without any default value,
-        it raises C{AttributeError} when trying to access it.
+        A no-op method with a mismatched prefix to not be discovered.
         """
-        self.assertRaises(AttributeError, getattr, self.tester, "x")
 
 
 
-class LookupsTestCase(unittest.TestCase):
+class AccumulateMethodsTests(TestCase):
+    """
+    Tests for L{accumulateMethods} which finds methods on a class hierarchy and
+    adds them to a dictionary.
+    """
+
+    def test_ownClass(self):
+        """
+        If x is and instance of Base and Base defines a method named method,
+        L{accumulateMethods} adds an item to the given dictionary with
+        C{"method"} as the key and a bound method object for Base.method value.
+        """
+        x = Base()
+        output = {}
+        accumulateMethods(x, output)
+        self.assertEqual({"method": x.method}, output)
+
+
+    def test_baseClass(self):
+        """
+        If x is an instance of Sub and Sub is a subclass of Base and Base
+        defines a method named method, L{accumulateMethods} adds an item to the
+        given dictionary with C{"method"} as the key and a bound method object
+        for Base.method as the value.
+        """
+        x = Sub()
+        output = {}
+        accumulateMethods(x, output)
+        self.assertEqual({"method": x.method}, output)
+
+
+    def test_prefix(self):
+        """
+        If a prefix is given, L{accumulateMethods} limits its results to
+        methods beginning with that prefix.  Keys in the resulting dictionary
+        also have the prefix removed from them.
+        """
+        x = Separate()
+        output = {}
+        accumulateMethods(x, output, 'good_')
+        self.assertEqual({'method': x.good_method}, output)
+
+
+
+class PrefixedMethodsTests(TestCase):
+    """
+    Tests for L{prefixedMethods} which finds methods on a class hierarchy and
+    adds them to a dictionary.
+    """
+
+    def test_onlyObject(self):
+        """
+        L{prefixedMethods} returns a list of the methods discovered on an
+        object.
+        """
+        x = Base()
+        output = prefixedMethods(x)
+        self.assertEqual([x.method], output)
+
+
+    def test_prefix(self):
+        """
+        If a prefix is given, L{prefixedMethods} returns only methods named
+        with that prefix.
+        """
+        x = Separate()
+        output = prefixedMethods(x, 'good_')
+        self.assertEqual([x.good_method], output)
+
+
+
+class PrefixedMethodNamesTests(TestCase):
+    """
+    Tests for L{prefixedMethodNames}.
+    """
+    def test_method(self):
+        """
+        L{prefixedMethodNames} returns a list including methods with the given
+        prefix defined on the class passed to it.
+        """
+        self.assertEqual(["method"], prefixedMethodNames(Separate, "good_"))
+
+
+    def test_inheritedMethod(self):
+        """
+        L{prefixedMethodNames} returns a list included methods with the given
+        prefix defined on base classes of the class passed to it.
+        """
+        class Child(Separate):
+            pass
+        self.assertEqual(["method"], prefixedMethodNames(Child, "good_"))
+
+
+
+class AddMethodNamesToDictTests(TestCase):
+    """
+    Tests for L{addMethodNamesToDict}.
+    """
+    def test_baseClass(self):
+        """
+        If C{baseClass} is passed to L{addMethodNamesToDict}, only methods which
+        are a subclass of C{baseClass} are added to the result dictionary.
+        """
+        class Alternate(object):
+            pass
+
+        class Child(Separate, Alternate):
+            def good_alternate(self):
+                pass
+
+        result = {}
+        addMethodNamesToDict(Child, result, 'good_', Alternate)
+        self.assertEqual({'alternate': 1}, result)
+
+
+
+class Summer(object):
+    """
+    A class we look up as part of the LookupsTestCase.
+    """
+
+    def reallySet(self):
+        """
+        Do something.
+        """
+
+
+
+class LookupsTestCase(TestCase):
     """
     Tests for L{namedClass}, L{namedModule}, and L{namedAny}.
     """
@@ -147,8 +197,8 @@ class LookupsTestCase(unittest.TestCase):
         L{namedClass} should return the class object for the name it is passed.
         """
         self.assertIdentical(
-            reflect.namedClass("twisted.python.reflect.Summer"),
-            reflect.Summer)
+            reflect.namedClass("twisted.test.test_reflect.Summer"),
+            Summer)
 
 
     def test_namedModuleLookup(self):
@@ -156,8 +206,9 @@ class LookupsTestCase(unittest.TestCase):
         L{namedModule} should return the module object for the name it is
         passed.
         """
+        from twisted.python import monkey
         self.assertIdentical(
-            reflect.namedModule("twisted.python.reflect"), reflect)
+            reflect.namedModule("twisted.python.monkey"), monkey)
 
 
     def test_namedAnyPackageLookup(self):
@@ -168,12 +219,14 @@ class LookupsTestCase(unittest.TestCase):
         self.assertIdentical(
             reflect.namedAny("twisted.python"), twisted.python)
 
+
     def test_namedAnyModuleLookup(self):
         """
         L{namedAny} should return the module object for the name it is passed.
         """
+        from twisted.python import monkey
         self.assertIdentical(
-            reflect.namedAny("twisted.python.reflect"), reflect)
+            reflect.namedAny("twisted.python.monkey"), monkey)
 
 
     def test_namedAnyClassLookup(self):
@@ -181,7 +234,8 @@ class LookupsTestCase(unittest.TestCase):
         L{namedAny} should return the class object for the name it is passed.
         """
         self.assertIdentical(
-            reflect.namedAny("twisted.python.reflect.Summer"), reflect.Summer)
+            reflect.namedAny("twisted.test.test_reflect.Summer"),
+            Summer)
 
 
     def test_namedAnyAttributeLookup(self):
@@ -193,8 +247,9 @@ class LookupsTestCase(unittest.TestCase):
         # object every time.  This is a foolishness of Python's object
         # implementation, not a bug in Twisted.
         self.assertEqual(
-            reflect.namedAny("twisted.python.reflect.Summer.reallySet"),
-            reflect.Summer.reallySet)
+            reflect.namedAny(
+                "twisted.test.test_reflect.Summer.reallySet"),
+            Summer.reallySet)
 
 
     def test_namedAnySecondAttributeLookup(self):
@@ -205,8 +260,9 @@ class LookupsTestCase(unittest.TestCase):
         """
         self.assertIdentical(
             reflect.namedAny(
-                "twisted.python.reflect.Summer.reallySet.__doc__"),
-            reflect.Summer.reallySet.__doc__)
+                "twisted.test.test_reflect."
+                "Summer.reallySet.__doc__"),
+            Summer.reallySet.__doc__)
 
 
     def test_importExceptions(self):
@@ -217,15 +273,15 @@ class LookupsTestCase(unittest.TestCase):
         self.assertRaises(
             ZeroDivisionError,
             reflect.namedAny, "twisted.test.reflect_helper_ZDE")
-        # Make sure that this behavior is *consistent* for 2.3, where there is
-        # no post-failed-import cleanup
+        # Make sure that there is post-failed-import cleanup
         self.assertRaises(
             ZeroDivisionError,
             reflect.namedAny, "twisted.test.reflect_helper_ZDE")
         self.assertRaises(
             ValueError,
             reflect.namedAny, "twisted.test.reflect_helper_VE")
-        # Modules which themselves raise ImportError when imported should result in an ImportError
+        # Modules which themselves raise ImportError when imported should
+        # result in an ImportError
         self.assertRaises(
             ImportError,
             reflect.namedAny, "twisted.test.reflect_helper_IE")
@@ -248,17 +304,19 @@ class LookupsTestCase(unittest.TestCase):
             reflect.namedAny, "twisted.nosuch.modulein.theworld")
         self.assertRaises(
             AttributeError,
-            reflect.namedAny, "twisted.python.reflect.Summer.nosuchattributeintheworld")
+            reflect.namedAny,
+            "twisted.test.test_reflect.Summer.nosuchattribute")
 
 
     def test_invalidNames(self):
         """
         Passing a name which isn't a fully-qualified Python name to L{namedAny}
         should result in one of the following exceptions:
-        - L{InvalidName}: the name is not a dot-separated list of Python objects
-        - L{ObjectNotFound}: the object doesn't exist
-        - L{ModuleNotFound}: the object doesn't exist and there is only one
-          component in the name
+         - L{InvalidName}: the name is not a dot-separated list of Python
+           objects
+         - L{ObjectNotFound}: the object doesn't exist
+         - L{ModuleNotFound}: the object doesn't exist and there is only one
+           component in the name
         """
         err = self.assertRaises(reflect.ModuleNotFound, reflect.namedAny,
                                 'nosuchmoduleintheworld')
@@ -287,161 +345,40 @@ class LookupsTestCase(unittest.TestCase):
                 "identifiers, not %r" % (invalidName,))
 
 
-
-class ImportHooksLookupTests(LookupsTestCase):
-    """
-    Tests for lookup methods in the presence of L{ihooks}-style import hooks.
-    Runs all of the tests from L{LookupsTestCase} after installing a custom
-    import hook.
-    """
-    def setUp(self):
+    def test_requireModuleImportError(self):
         """
-        Perturb the normal import behavior subtly by installing an import
-        hook.  No custom behavior is provided, but this adds some extra
-        frames to the call stack, which L{namedAny} must be able to account
-        for.
+        When module import fails with ImportError it returns the specified
+        default value.
         """
-        self.importer = ModuleImporter()
-        self.importer.install()
+        for name in ['nosuchmtopodule', 'no.such.module']:
+            default = object()
+
+            result = reflect.requireModule(name, default=default)
+
+            self.assertIs(result, default)
 
 
-    def tearDown(self):
+    def test_requireModuleDefaultNone(self):
         """
-        Uninstall the custom import hook.
+        When module import fails it returns C{None} by default.
         """
-        self.importer.uninstall()
+        result = reflect.requireModule('no.such.module')
+
+        self.assertIs(None, result)
 
 
-
-class ObjectGrep(unittest.TestCase):
-    def test_dictionary(self):
+    def test_requireModuleRequestedImport(self):
         """
-        Test references search through a dictionnary, as a key or as a value.
+        When module import succeed it returns the module and not the default
+        value.
         """
-        o = object()
-        d1 = {None: o}
-        d2 = {o: None}
+        from twisted.python import monkey
+        default = object()
 
-        self.assertIn("[None]", reflect.objgrep(d1, o, reflect.isSame))
-        self.assertIn("{None}", reflect.objgrep(d2, o, reflect.isSame))
-
-    def test_list(self):
-        """
-        Test references search through a list.
-        """
-        o = object()
-        L = [None, o]
-
-        self.assertIn("[1]", reflect.objgrep(L, o, reflect.isSame))
-
-    def test_tuple(self):
-        """
-        Test references search through a tuple.
-        """
-        o = object()
-        T = (o, None)
-
-        self.assertIn("[0]", reflect.objgrep(T, o, reflect.isSame))
-
-    def test_instance(self):
-        """
-        Test references search through an object attribute.
-        """
-        class Dummy:
-            pass
-        o = object()
-        d = Dummy()
-        d.o = o
-
-        self.assertIn(".o", reflect.objgrep(d, o, reflect.isSame))
-
-    def test_weakref(self):
-        """
-        Test references search through a weakref object.
-        """
-        class Dummy:
-            pass
-        o = Dummy()
-        w1 = weakref.ref(o)
-
-        self.assertIn("()", reflect.objgrep(w1, o, reflect.isSame))
-
-    def test_boundMethod(self):
-        """
-        Test references search through method special attributes.
-        """
-        class Dummy:
-            def dummy(self):
-                pass
-        o = Dummy()
-        m = o.dummy
-
-        self.assertIn(".im_self", reflect.objgrep(m, m.im_self, reflect.isSame))
-        self.assertIn(".im_class", reflect.objgrep(m, m.im_class, reflect.isSame))
-        self.assertIn(".im_func", reflect.objgrep(m, m.im_func, reflect.isSame))
-
-    def test_everything(self):
-        """
-        Test references search using complex set of objects.
-        """
-        class Dummy:
-            def method(self):
-                pass
-
-        o = Dummy()
-        D1 = {(): "baz", None: "Quux", o: "Foosh"}
-        L = [None, (), D1, 3]
-        T = (L, {}, Dummy())
-        D2 = {0: "foo", 1: "bar", 2: T}
-        i = Dummy()
-        i.attr = D2
-        m = i.method
-        w = weakref.ref(m)
-
-        self.assertIn("().im_self.attr[2][0][2]{'Foosh'}", reflect.objgrep(w, o, reflect.isSame))
-
-    def test_depthLimit(self):
-        """
-        Test the depth of references search.
-        """
-        a = []
-        b = [a]
-        c = [a, b]
-        d = [a, c]
-
-        self.assertEqual(['[0]'], reflect.objgrep(d, a, reflect.isSame, maxDepth=1))
-        self.assertEqual(['[0]', '[1][0]'], reflect.objgrep(d, a, reflect.isSame, maxDepth=2))
-        self.assertEqual(['[0]', '[1][0]', '[1][1][0]'], reflect.objgrep(d, a, reflect.isSame, maxDepth=3))
-
-    def test_deque(self):
-        """
-        Test references search through a deque object. Only for Python > 2.3.
-        """
-        o = object()
-        D = deque()
-        D.append(None)
-        D.append(o)
-
-        self.assertIn("[1]", reflect.objgrep(D, o, reflect.isSame))
-
-    if deque is None:
-        test_deque.skip = "Deque not available"
-
-
-class GetClass(unittest.TestCase):
-    def testOld(self):
-        class OldClass:
-            pass
-        old = OldClass()
-        self.assertIn(reflect.getClass(OldClass).__name__, ('class', 'classobj'))
-        self.assertEqual(reflect.getClass(old).__name__, 'OldClass')
-
-    def testNew(self):
-        class NewClass(object):
-            pass
-        new = NewClass()
-        self.assertEqual(reflect.getClass(NewClass).__name__, 'type')
-        self.assertEqual(reflect.getClass(new).__name__, 'NewClass')
+        self.assertIs(
+            reflect.requireModule('twisted.python.monkey', default=default),
+            monkey,
+            )
 
 
 
@@ -456,6 +393,7 @@ class Breakable(object):
         else:
             return '<Breakable>'
 
+
     def __repr__(self):
         if self.breakRepr:
             raise RuntimeError("repr!")
@@ -475,10 +413,9 @@ class BrokenType(Breakable, type):
 
 
 
-class BTBase(Breakable):
-    __metaclass__ = BrokenType
-    breakRepr = True
-    breakStr = True
+BTBase = BrokenType('BTBase', (Breakable,),
+                    {"breakRepr": True,
+                     "breakStr": True})
 
 
 
@@ -487,7 +424,7 @@ class NoClassAttr(Breakable):
 
 
 
-class SafeRepr(unittest.TestCase):
+class SafeRepr(TestCase):
     """
     Tests for L{reflect.safe_repr} function.
     """
@@ -532,24 +469,20 @@ class SafeRepr(unittest.TestCase):
         reflect.safe_repr(X())
 
 
-    def test_unsignedID(self):
+    def test_brokenReprIncludesID(self):
         """
-        L{unsignedID} is used to print ID of the object in case of error, not
-        standard ID value which can be negative.
+        C{id} is used to print the ID of the object in case of an error.
+
+        L{safe_repr} includes a traceback after a newline, so we only check
+        against the first line of the repr.
         """
         class X(BTBase):
             breakRepr = True
 
-        ids = {X: 100}
-        def fakeID(obj):
-            try:
-                return ids[obj]
-            except (TypeError, KeyError):
-                return id(obj)
-        self.addCleanup(util.setIDFunction, util.setIDFunction(fakeID))
-
         xRepr = reflect.safe_repr(X)
-        self.assertIn("0x64", xRepr)
+        xReprExpected = ('<BrokenType instance at 0x%x with repr error:'
+                         % (id(X),))
+        self.assertEqual(xReprExpected, xRepr.split('\n')[0])
 
 
     def test_brokenClassStr(self):
@@ -588,7 +521,7 @@ class SafeRepr(unittest.TestCase):
 
 
 
-class SafeStr(unittest.TestCase):
+class SafeStr(TestCase):
     """
     Tests for L{reflect.safe_str} function.
     """
@@ -653,33 +586,57 @@ class SafeStr(unittest.TestCase):
 
 
 
-class FilenameToModule(unittest.TestCase):
+class FilenameToModule(TestCase):
     """
-    Test L{reflect.filenameToModuleName} detection.
+    Test L{filenameToModuleName} detection.
     """
+
+    def setUp(self):
+        self.path = os.path.join(self.mktemp(), "fakepackage", "test")
+        os.makedirs(self.path)
+        with open(os.path.join(self.path, "__init__.py"), "w") as f:
+            f.write("")
+        with open(os.path.join(os.path.dirname(self.path), "__init__.py"),
+                  "w") as f:
+            f.write("")
+
+
     def test_directory(self):
         """
-        Tests it finds good name for directories/packages.
+        L{filenameToModuleName} returns the correct module (a package) given a
+        directory.
         """
-        module = reflect.filenameToModuleName(os.path.join('twisted', 'test'))
-        self.assertEqual(module, 'test')
-        module = reflect.filenameToModuleName(os.path.join('twisted', 'test')
-                                              + os.path.sep)
-        self.assertEqual(module, 'test')
+        module = reflect.filenameToModuleName(self.path)
+        self.assertEqual(module, 'fakepackage.test')
+        module = reflect.filenameToModuleName(self.path + os.path.sep)
+        self.assertEqual(module, 'fakepackage.test')
+
 
     def test_file(self):
         """
-        Test it finds good name for files.
+        L{filenameToModuleName} returns the correct module given the path to
+        its file.
         """
         module = reflect.filenameToModuleName(
-            os.path.join('twisted', 'test', 'test_reflect.py'))
-        self.assertEqual(module, 'test_reflect')
+            os.path.join(self.path, 'test_reflect.py'))
+        self.assertEqual(module, 'fakepackage.test.test_reflect')
+
+
+    def test_bytes(self):
+        """
+        L{filenameToModuleName} returns the correct module given a C{bytes}
+        path to its file.
+        """
+        module = reflect.filenameToModuleName(
+            os.path.join(self.path.encode("utf-8"), b'test_reflect.py'))
+        # Module names are always native string:
+        self.assertEqual(module, 'fakepackage.test.test_reflect')
 
 
 
-class FullyQualifiedNameTests(unittest.TestCase):
+class FullyQualifiedNameTests(TestCase):
     """
-    Test for L{reflect.fullyQualifiedName}.
+    Test for L{fullyQualifiedName}.
     """
 
     def _checkFullyQualifiedName(self, obj, expected):
@@ -687,14 +644,13 @@ class FullyQualifiedNameTests(unittest.TestCase):
         Helper to check that fully qualified name of C{obj} results to
         C{expected}.
         """
-        self.assertEqual(
-            reflect.fullyQualifiedName(obj), expected)
+        self.assertEqual(fullyQualifiedName(obj), expected)
 
 
     def test_package(self):
         """
-        L{reflect.fullyQualifiedName} returns the full name of a package and
-        a subpackage.
+        L{fullyQualifiedName} returns the full name of a package and a
+        subpackage.
         """
         import twisted
         self._checkFullyQualifiedName(twisted, 'twisted')
@@ -704,164 +660,241 @@ class FullyQualifiedNameTests(unittest.TestCase):
 
     def test_module(self):
         """
-        L{reflect.fullyQualifiedName} returns the name of a module inside a a
-        package.
+        L{fullyQualifiedName} returns the name of a module inside a a package.
         """
-        self._checkFullyQualifiedName(reflect, 'twisted.python.reflect')
-        import twisted.trial.unittest
-        self._checkFullyQualifiedName(twisted.trial.unittest,
-                                      'twisted.trial.unittest')
+        import twisted.python.compat
+        self._checkFullyQualifiedName(
+            twisted.python.compat, 'twisted.python.compat')
 
 
     def test_class(self):
         """
-        L{reflect.fullyQualifiedName} returns the name of a class and its
-        module.
+        L{fullyQualifiedName} returns the name of a class and its module.
         """
-        self._checkFullyQualifiedName(reflect.Settable,
-                                      'twisted.python.reflect.Settable')
+        self._checkFullyQualifiedName(
+            FullyQualifiedNameTests,
+            '%s.FullyQualifiedNameTests' % (__name__,))
 
 
     def test_function(self):
         """
-        L{reflect.fullyQualifiedName} returns the name of a function inside its
-        module.
+        L{fullyQualifiedName} returns the name of a function inside its module.
         """
-        self._checkFullyQualifiedName(reflect.fullyQualifiedName,
-            "twisted.python.reflect.fullyQualifiedName")
+        self._checkFullyQualifiedName(
+            fullyQualifiedName, "twisted.python.reflect.fullyQualifiedName")
 
 
     def test_boundMethod(self):
         """
-        L{reflect.fullyQualifiedName} returns the name of a bound method inside
-        its class and its module.
+        L{fullyQualifiedName} returns the name of a bound method inside its
+        class and its module.
         """
         self._checkFullyQualifiedName(
-            reflect.PropertyAccessor().reallyDel,
-            "twisted.python.reflect.PropertyAccessor.reallyDel")
+            self.test_boundMethod,
+            "%s.%s.test_boundMethod" % (__name__, self.__class__.__name__))
 
 
     def test_unboundMethod(self):
         """
-        L{reflect.fullyQualifiedName} returns the name of an unbound method
-        inside its class and its module.
+        L{fullyQualifiedName} returns the name of an unbound method inside its
+        class and its module.
         """
         self._checkFullyQualifiedName(
-            reflect.PropertyAccessor.reallyDel,
-            "twisted.python.reflect.PropertyAccessor.reallyDel")
+            self.__class__.test_unboundMethod,
+            "%s.%s.test_unboundMethod" % (__name__, self.__class__.__name__))
 
 
+class ObjectGrep(unittest.TestCase):
+    if _PY3:
+        # This is to be removed when fixing #6986
+        skip = "twisted.python.reflect.objgrep hasn't been ported to Python 3"
 
-class DeprecationTestCase(unittest.TestCase):
-    """
-    Test deprecations in twisted.python.reflect
-    """
 
-    def test_allYourBase(self):
+    def test_dictionary(self):
         """
-        Test deprecation of L{reflect.allYourBase}. See #5481 for removal.
+        Test references search through a dictionnary, as a key or as a value.
         """
-        self.callDeprecated(
-            (Version("Twisted", 11, 0, 0), "inspect.getmro"),
-            reflect.allYourBase, DeprecationTestCase)
+        o = object()
+        d1 = {None: o}
+        d2 = {o: None}
 
+        self.assertIn("[None]", reflect.objgrep(d1, o, reflect.isSame))
+        self.assertIn("{None}", reflect.objgrep(d2, o, reflect.isSame))
 
-    def test_accumulateBases(self):
+    def test_list(self):
         """
-        Test deprecation of L{reflect.accumulateBases}. See #5481 for removal.
+        Test references search through a list.
         """
-        l = []
-        self.callDeprecated(
-            (Version("Twisted", 11, 0, 0), "inspect.getmro"),
-            reflect.accumulateBases, DeprecationTestCase, l, None)
+        o = object()
+        L = [None, o]
 
+        self.assertIn("[1]", reflect.objgrep(L, o, reflect.isSame))
 
-    def lookForDeprecationWarning(self, testMethod, attributeName, warningMsg):
+    def test_tuple(self):
         """
-        Test deprecation of attribute 'reflect.attributeName' by calling
-        'reflect.testMethod' and verifying the warning message
-        'reflect.warningMsg'
-
-        @param testMethod: Name of the offending function to be used with
-            flushWarnings
-        @type testmethod: C{str}
-
-        @param attributeName: Name of attribute to be checked for deprecation
-        @type attributeName: C{str}
-
-        @param warningMsg: Deprecation warning message
-        @type warningMsg: C{str}
+        Test references search through a tuple.
         """
-        warningsShown = self.flushWarnings([testMethod])
-        self.assertEqual(len(warningsShown), 1)
-        self.assertIdentical(warningsShown[0]['category'], DeprecationWarning)
-        self.assertEqual(
-            warningsShown[0]['message'],
-            "twisted.python.reflect." + attributeName + " "
-            "was deprecated in Twisted 12.1.0: " + warningMsg + ".")
+        o = object()
+        T = (o, None)
+
+        self.assertIn("[0]", reflect.objgrep(T, o, reflect.isSame))
+
+    def test_instance(self):
+        """
+        Test references search through an object attribute.
+        """
+        class Dummy:
+            pass
+        o = object()
+        d = Dummy()
+        d.o = o
+
+        self.assertIn(".o", reflect.objgrep(d, o, reflect.isSame))
+
+    def test_weakref(self):
+        """
+        Test references search through a weakref object.
+        """
+        class Dummy:
+            pass
+        o = Dummy()
+        w1 = weakref.ref(o)
+
+        self.assertIn("()", reflect.objgrep(w1, o, reflect.isSame))
+
+    def test_boundMethod(self):
+        """
+        Test references search through method special attributes.
+        """
+        class Dummy:
+            def dummy(self):
+                pass
+        o = Dummy()
+        m = o.dummy
+
+        self.assertIn(".__self__",
+                      reflect.objgrep(m, m.__self__, reflect.isSame))
+        self.assertIn(".__self__.__class__",
+                      reflect.objgrep(m, m.__self__.__class__, reflect.isSame))
+        self.assertIn(".__func__",
+                      reflect.objgrep(m, m.__func__, reflect.isSame))
+
+    def test_everything(self):
+        """
+        Test references search using complex set of objects.
+        """
+        class Dummy:
+            def method(self):
+                pass
+
+        o = Dummy()
+        D1 = {(): "baz", None: "Quux", o: "Foosh"}
+        L = [None, (), D1, 3]
+        T = (L, {}, Dummy())
+        D2 = {0: "foo", 1: "bar", 2: T}
+        i = Dummy()
+        i.attr = D2
+        m = i.method
+        w = weakref.ref(m)
+
+        self.assertIn("().__self__.attr[2][0][2]{'Foosh'}",
+                      reflect.objgrep(w, o, reflect.isSame))
+
+    def test_depthLimit(self):
+        """
+        Test the depth of references search.
+        """
+        a = []
+        b = [a]
+        c = [a, b]
+        d = [a, c]
+
+        self.assertEqual(['[0]'], reflect.objgrep(d, a, reflect.isSame, maxDepth=1))
+        self.assertEqual(['[0]', '[1][0]'], reflect.objgrep(d, a, reflect.isSame, maxDepth=2))
+        self.assertEqual(['[0]', '[1][0]', '[1][1][0]'], reflect.objgrep(d, a, reflect.isSame, maxDepth=3))
 
 
-    def test_settable(self):
+    def test_deque(self):
         """
-        Test deprecation of L{reflect.Settable}.
+        Test references search through a deque object.
         """
-        reflect.Settable()
-        self.lookForDeprecationWarning(
-            self.test_settable, "Settable",
-            "Settable is old and untested. Please write your own version of this "
-            "functionality if you need it")
+        o = object()
+        D = deque()
+        D.append(None)
+        D.append(o)
+
+        self.assertIn("[1]", reflect.objgrep(D, o, reflect.isSame))
 
 
-    def test_accessorType(self):
-        """
-        Test deprecation of L{reflect.AccessorType}.
-        """
-        reflect.AccessorType(' ', ( ), { })
-        self.lookForDeprecationWarning(
-            self.test_accessorType, "AccessorType",
-            "AccessorType is old and untested. Please write your own version of "
-            "this functionality if you need it")
+class GetClass(unittest.TestCase):
+    if _PY3:
+        oldClassNames = ['type']
+    else:
+        oldClassNames = ['class', 'classobj']
+
+    def testOld(self):
+        class OldClass:
+            pass
+        old = OldClass()
+        self.assertIn(reflect.getClass(OldClass).__name__, self.oldClassNames)
+        self.assertEqual(reflect.getClass(old).__name__, 'OldClass')
+
+    def testNew(self):
+        class NewClass(object):
+            pass
+        new = NewClass()
+        self.assertEqual(reflect.getClass(NewClass).__name__, 'type')
+        self.assertEqual(reflect.getClass(new).__name__, 'NewClass')
 
 
-    def test_propertyAccessor(self):
+if not _PY3:
+    # The functions tested below are deprecated but still used by external
+    # projects like Nevow 0.10. They are not going to be ported to Python 3
+    # (hence the condition above) and will be removed as soon as no project used
+    # by Twisted will depend on these functions. Also, have a look at the
+    # comments related to those functions in twisted.python.reflect.
+    class DeprecationTestCase(unittest.TestCase):
         """
-        Test deprecation of L{reflect.PropertyAccessor}.
+        Test deprecations in twisted.python.reflect
         """
-        reflect.PropertyAccessor()
-        self.lookForDeprecationWarning(
-            self.test_propertyAccessor, "PropertyAccessor",
-            "PropertyAccessor is old and untested. Please write your own "
-            "version of this functionality if you need it")
+
+        def test_allYourBase(self):
+            """
+            Test deprecation of L{reflect.allYourBase}. See #5481 for removal.
+            """
+            self.callDeprecated(
+                (Version("Twisted", 11, 0, 0), "inspect.getmro"),
+                reflect.allYourBase, DeprecationTestCase)
 
 
-    def test_accessor(self):
-        """
-        Test deprecation of L{reflect.Accessor}.
-        """
-        reflect.Accessor()
-        self.lookForDeprecationWarning(
-            self.test_accessor, "Accessor",
-            "Accessor is an implementation for Python 2.1 which is no longer "
-            "supported by Twisted")
+        def test_accumulateBases(self):
+            """
+            Test deprecation of L{reflect.accumulateBases}. See #5481 for removal.
+            """
+            l = []
+            self.callDeprecated(
+                (Version("Twisted", 11, 0, 0), "inspect.getmro"),
+                reflect.accumulateBases, DeprecationTestCase, l, None)
 
 
-    def test_originalAccessor(self):
-        """
-        Test deprecation of L{reflect.OriginalAccessor}.
-        """
-        reflect.OriginalAccessor()
-        self.lookForDeprecationWarning(
-            self.test_originalAccessor, "OriginalAccessor",
-            "OriginalAccessor is a reference to class "
-            "twisted.python.reflect.Accessor which is deprecated")
+        def test_getcurrent(self):
+            """
+            Test deprecation of L{reflect.getcurrent}.
+            """
+
+            class C:
+                pass
+
+            self.callDeprecated(
+                Version("Twisted", 14, 0, 0),
+                reflect.getcurrent, C)
 
 
-    def test_summer(self):
-        """
-        Test deprecation of L{reflect.Summer}.
-        """
-        reflect.Summer()
-        self.lookForDeprecationWarning(
-            self.test_summer, "Summer",
-            "Summer is a child class of twisted.python.reflect.Accessor which "
-            "is deprecated")
+        def test_isinst(self):
+            """
+            Test deprecation of L{reflect.isinst}.
+            """
+
+            self.callDeprecated(
+                (Version("Twisted", 14, 0, 0), "isinstance"),
+                reflect.isinst, object(), object)
